package cologne.eck.all_peas.control;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.HashSet;
import java.util.Locale;
import java.util.MissingResourceException;
import java.util.PropertyResourceBundle;
import java.util.ResourceBundle;

import cologne.eck.all_peas.data.PeaProperties;
import cologne.eck.all_peas.vm_specific.JREBridge;
import cologne.eck.all_peas.vm_specific.JREProperties;
import cologne.eck.tools.Converter;
import cologne.eck.tools.TestLog;

public class Translator {
	
	/**
	 * The bundle name for all PEAs
	 */
	private static final String bundleFileName = "PeaLanguagesBundle";
	/**
	 * The bundle name for Calendar Lock PEA
	 */
	private static final String calBundleFileName = "CalendarLanguageBundle";
	
	/**
	 * The last successfully set language bundle for all PEAs
	 */
	private ResourceBundle lastBundle = null;	
	/**
	 * The last successfully set language bundle for all PEAs
	 */
	private ResourceBundle lastPeaBundle = null;
	
	
	public String translate(String key, String specificPEA) {
		
		ResourceBundle usedBundle = null;
		if (specificPEA == null) { // bundle for all PEAs
			if (lastBundle == null) {
				try {
					TestLog.v(getClass(), "Set language support");
					boolean setBundleSuccess = setLanguagesBundle(PeaProperties.getLastLanguage());
					if (setBundleSuccess == false) {
						TestLog.ve(getClass(), "Can't set language bundle...", 5);
					}
				} catch (MalformedURLException e) {
					TestLog.ve(getClass(), e.getLocalizedMessage() + " - " + e.toString(), 5);
				}
			}
			usedBundle = lastBundle;//(ResourceBundle)JREProperties.getBundle();
		} else if (specificPEA.equals("calendar")) {
			 usedBundle = lastPeaBundle;
		} // TODO other PEAs


		if ( JREProperties.getBundle() != null) {
			if (usedBundle == null) {
				try { // should not happen, should have been set before with setLanguagesBundle()
					TestLog.ve(getClass(), "Set language support for specific PEA",5);
					usedBundle = getPeaLanguageBundle(PeaProperties.getLastLanguage());//setLanguagesBundle(null);
					lastPeaBundle = usedBundle;
				} catch (Exception e) {
					TestLog.ve(getClass(), e.getLocalizedMessage() + " - " + e.toString(), 5);
				}
			}
			if (usedBundle != null &&  usedBundle.containsKey(key) ){
				return  usedBundle.getString(key);		
			} else {
				// Something went wrong... try to use default language (English) or as fallback return the key
				String bundleName = null;
				if (specificPEA == null) { // bundle for all PEAs
					bundleName = bundleFileName;
				} else if (specificPEA.equals("calendar")) {
					bundleName = calBundleFileName;
				}
				// get value from default file inside jar
				try {
					TestLog.ve(JREBridge.class, PeaProperties.getLastLanguage() + " - Cant find value for key " + key + ", search in default bundle...", 3);
					//ResourceBundle bundle  = new PropertyResourceBundle(JREBridge.class.getResourceAsStream("/resources/PeaLanguagesBundle.properties"));
					ResourceBundle bundle  = getBundleFromInsideJar("", bundleName);

					if (bundle != null) { // check 
						return bundle.getString(key);
					} else {
						TestLog.ve(JREBridge.class, "Could not load default bundle from inside jar");
						// get default bundle from folder
						bundle  = getBundleFromExternFolder("", bundleName);
						if (bundle != null) {
							return bundle.getString(key);
						} else {
							return key;
						}
					}
				} catch (Exception e) {
					TestLog.ve(JREBridge.class, "Cant find value for key " + key + " in default bundle inside jar", 5);
				}					
				return key;
			}
		} else {
			TestLog.e(JREBridge.class, "Invalid bundle, value not found for " + key, 5);
			return key;
		}
	}

	/**
	 * Translate using the calendar language bundle
	 * 
	 * @param key	the key to translate
	 * @return	the translated key or the key if no matching key is found
	 */
/*	public final static String calTranslate(String key) {
		if (lastPeaBundle == null) {
			try {
				CalendarControl.setLanguagesBundle(null);
			} catch (MalformedURLException e) {
				TestLog.e(CalendarSetting.class, "Language support failed: "  + e.toString() + "\n  " + e.getMessage());
			}
			if (lastPeaBundle == null) {
				return key;
			} else {
				String value = null;
				try {
					value = lastPeaBundle.getString(key);
				} catch (Exception e) {
					try {
						TestLog.ve(CalendarSetting.class, "Cant find value for key " + key + ", search in default bundle...");
						ResourceBundle bundle  = new PropertyResourceBundle(JREBridge.class.getResourceAsStream("/resources/CalendarLanguageBundle.properties"));
						return bundle.getString(key);
					} catch (Exception e1) {
						TestLog.ve(CalendarSetting.class, "Cant find value for key " + key + " in default bundle inside jar", 5);
					}		
					TestLog.ve(CalendarSetting.class, "Missing key: " + key + " in calendar bundle " + lastPeaBundle.getLocale() + " - " + lastPeaBundle.getBaseBundleName());
					value = key;
				}
				return value;
			}
		} else {
			String value = null;
			try {
				value = lastPeaBundle.getString(key);
			} catch (Exception e) {
				try {
					TestLog.ve(CalendarSetting.class, "Cant find value for key " + key + ", search in default bundle...");
					ResourceBundle bundle  = new PropertyResourceBundle(JREBridge.class.getResourceAsStream("/resources/CalendarLanguageBundle.properties"));
					return bundle.getString(key);
				} catch (Exception e1) {
					TestLog.ve(CalendarSetting.class, "Cant find value for key " + key + " in default bundle inside jar", 5);
				}	
				TestLog.ve(CalendarSetting.class, "Missing key: " + key + " in calendar bundle " + lastPeaBundle.getLocale() + " - " + lastPeaBundle.getBaseBundleName());
				value = key;
			}
			return value;
		}		
	}
*/


	public final String[] getAvailableLanguages()  {
		HashSet<String> languagesSet = new HashSet<String>();
		// add default language en
		languagesSet.add("en");
		
		// get bundles from resource directory in file system:
		File resDir = new File(System.getProperty("user.dir") + File.separator 
				+ "resources");
		if (resDir.exists() && resDir.isDirectory()) {
			File[] resList = resDir.listFiles();
			if (resList != null) {
				for (File f : resList) {
					if (f != null) {
						if (f.isDirectory()) {
							File[] langDir = f.listFiles();
							if (langDir == null) {
								continue;
							} else { // check for the properties file:
								String lang = f.getName();
								String fName = f.getAbsolutePath() + File.separator
										+ "PeaLanguagesBundle_" + lang + ".properties";
								TestLog.v(JREBridge.class, "Property files: " + fName);
								if ( new File(fName).exists()) {
									try {
										if ( new Locale(lang).getISO3Language() != null) {
											languagesSet.add(lang);
											TestLog.v(JREBridge.class, "bundle in resources found: " + lang);
										} else {
											TestLog.ve(JREBridge.class, "Invalid languages code: " + lang);
										}
									} catch (MissingResourceException e) {
										TestLog.ve(JREBridge.class, "Invalid languages code: " + lang);
									}									
								} else {
									TestLog.ve(JREBridge.class, "Not a language folder: " + lang);
								}
								// TODO if (PeaProperties.getFileType().equals("calendar")) {
							}
						}
					}
				}
			}
		} else {
			TestLog.v(JREBridge.class, "Missing folder or not a valid resource folder...");
		}
		
		// get bundles from inside jar: leading / required
		// Possible future languages (by countries where PEAs are most downloaded), for now only left-aligned
		String[] langs = {"de", "tr", "es", "fr", "ru", "bg", "cs", "da", "el", "et", "fi", "hu", "it", "nl", "pl", "pt", "ro", "sk", "sl", "sq", "sr", "sv",  };
		String pre = "/resources/";
		String mid = "/PeaLanguagesBundle_";
		String post = ".properties";
		for (String lang : langs) {
			if (getClass().getResourceAsStream(pre + lang + mid + lang + post) != null) {
				languagesSet.add(lang);
				TestLog.v(JREBridge.class, "bundle found: " + lang);
			}
		}

		return Converter.hashSetToArray(languagesSet);
	}

	/**
	 * Set the requested language bundle from a language String
	 * for all PEAs and if required for the specific PEA
	 * 
	 * @param _language		the language e.g. "en", "de", "tr"
	 * @throws MalformedURLException if the language String is not valid
	 * 
	 * @return	true for success
	 */
	public final boolean setLanguagesBundle(String _language) throws MalformedURLException {
		
		//ResourceBundle languageBundle = null;
		String langToReset = PeaProperties.getLastLanguage();
		//TestLog.e(getClass(), "Set language: " + _language, 5);

		try {
			Locale defaultLocale = Locale.getDefault();
			TestLog.v(JREBridge.class, "Set language: " + _language);
			String language = null;

			// i18n: 
			if (_language == null) {
				if (defaultLocale == null) { // openSuse
					defaultLocale = new Locale( System.getProperty("user.language") );
				}		
				language = defaultLocale.getLanguage(); // get default language
				if (language == null) { // fall back
					language = "";
				}
			} else {
				language = _language;
			}

			//======= 1. Check resource inside jar
			lastBundle = getBundleFromInsideJar(language, bundleFileName);

			if (lastBundle == null) {
				//======= 2. Check resource in external folder resources
				lastBundle = getBundleFromExternFolder(language, bundleFileName);
			}
			if (lastBundle == null) {
				lastBundle = ResourceBundle.getBundle("resources." + bundleFileName, new Locale(language));
			}
			if (lastBundle == null) {
				//======= 4. Fallback English
				lastBundle = getFallBackEnglish(bundleFileName);
			}		
			if (lastBundle == null && langToReset != null) {
				boolean success = setLanguagesBundle(langToReset);
				if (success == false) {
					PeaProperties.getVmBridge().exitWithError(JREProperties.getMainWindow(), "Language error", false);
				}
				language = langToReset;
				PeaProperties.setLastLanguage(language);
			}
			JREProperties.setBundle(lastBundle);
			PeaProperties.setLastLanguage(language);
		} catch (Exception e) {
			TestLog.ve(JREBridge.class, e.getLocalizedMessage() + " - " + e.toString());
			if (lastBundle == null && langToReset != null) {
				boolean success = setLanguagesBundle(langToReset);
				if (success == false) {
					PeaProperties.getVmBridge().exitWithError(JREProperties.getMainWindow(), "Language error", false);
					return false;
				}
			}		
		}
		
		// Now set the language bundle specific for the current PEA:
		// Try to use same language, if this fails, use English.
		// for now only Calendar Lock PEA
		if (PeaProperties.getFileType() != null) { // no called in Start.java
			if (PeaProperties.getFileType().equals("calendar")) {
				ResourceBundle b = getPeaLanguageBundle(PeaProperties.getLastLanguage());
				if (b == null) {
					return false;
				}
			}		
		}
		return true;
	}
	
	public ResourceBundle getPeaLanguageBundle(String language) {
		
		//String language = PeaProperties.getLastLanguage();
		// Now set the language bundle specific for the current PEA:
		// Try to use same language, if this fails, use English.
		// for now only Calendar Lock PEA
		if (PeaProperties.getFileType().equals("calendar")) {
			
			try {
				//======= 1. Check resource inside jar
				lastPeaBundle = getBundleFromInsideJar(language, calBundleFileName);

				if (lastPeaBundle == null) {
					//======= 2. Check resource in external folder resources
					lastPeaBundle = getBundleFromExternFolder(language, calBundleFileName);
				}
				if (lastPeaBundle == null) {
					lastPeaBundle = ResourceBundle.getBundle("resources." + calBundleFileName, new Locale(language));
				}
				if (lastPeaBundle == null) {
					//======= 4. Fallback English
					lastPeaBundle = getFallBackEnglish(calBundleFileName);
				}		
				if (lastPeaBundle == null) {
					PeaProperties.getVmBridge().exitWithError(JREProperties.getMainWindow(), "Language error", false);
					return null;
				}
			} catch (Exception e) {
				TestLog.ve(JREBridge.class, e.getLocalizedMessage() + " - " + e.toString());
				if (lastPeaBundle == null) {
					//======= 4. Fallback English
					lastPeaBundle = getFallBackEnglish(calBundleFileName);
				}	
			}			
			return lastPeaBundle;
		} else {
			TestLog.ve(getClass(), "Pea specific language bundle not yet supported...", 5);
			return null;
		}
	}
	
	private ResourceBundle getFallBackEnglish(String bundleFileName) {
		ResourceBundle result = null;
		result = getBundleFromInsideJar("", bundleFileName);
		if (result == null) {
			TestLog.ve(JREBridge.class, "Missing default bundle inside jar. ");// + Locale.forLanguageTag("en").toLanguageTag());
			result = getBundleFromExternFolder("", bundleFileName);
		}
		if (result == null) {
			TestLog.v(JREBridge.class, "Resource bundle Fallback English");
		}
		if (result == null) {
			TestLog.e(JREBridge.class, "No resource bundle English found");
			TestLog.ve(JREBridge.class, "No resource bundle English found", 9);
		}
		return result;
	}
		
	/**
	 * Get the language bundle from the external folder resources. 
	 * The bundle is stored in the sub-directory, named as the country code, e.g. "tr"
	 * 
	 * @param newLanguage	the country code, e.g. "tr" or null (for default)
	 * @param bundleName	the base name of the bundle filename without country code
	 * 
	 * @return	the resource bundle or null, if the corresponding bundle was not found
	 */
	private ResourceBundle getBundleFromExternFolder(String newLanguage, String bundleName) {
		ResourceBundle result = null;
		TestLog.v(JREBridge.class, "Read from external file...");
		// handle the default bundle en:
		if (newLanguage.equals("en")) {
			newLanguage = ""; // This is the default bundle
		}
		String underscore = "_";
		if (newLanguage == null || newLanguage.length() == 0) {
			underscore = "";
		}
		String fileName = "resources" + File.separator + newLanguage + File.separator + bundleName + underscore + newLanguage + ".properties";
		if (new File(fileName).exists()) {
			TestLog.v(JREBridge.class, "Bundle found in folder resources");
		} else {
			TestLog.v(JREBridge.class, "External Resource Bundle "  + fileName + " does not exist");
			return null;
		}
		try {
			result = new  PropertyResourceBundle(Files.newInputStream(Paths.get(fileName)));
					//"resources" + File.separator + language + File.separator + "PeaLanguagesBundle_" + language + ".properties")));
		} catch (IOException e) {
			TestLog.v(JREBridge.class, fileName + ": " +  e.toString() + " - " + e.getMessage());
		} catch (Exception e) {
			TestLog.v(JREBridge.class, fileName + ": " +  e.toString() + " - " + e.getMessage());
		}
		if (result != null) {
			TestLog.v(JREBridge.class, "Resource bundle in folder resources: " + newLanguage);
		}
		return result;		
	}
	
	/**
	 * Get the language bundle from inside the jar archive. 
	 * The bundle is stored in the sub-directory or the folder resources,
	 *  named as the country code, e.g. "tr"
	 * 
	 * @param newLanguage	the country code, e.g. "tr" or null (for default)
	 * @param bundleName	the base name of the bundle filename without country code
	 * 
	 * @return	the resource bundle or null, if the corresponding bundle was not found
	 */
	private ResourceBundle getBundleFromInsideJar(String newLanguage, String bundleName) {
		ResourceBundle result = null;
		
		// Handle the default bundle en
		if (newLanguage.equals("en")) {
			newLanguage = ""; // This is the default bundle
		}
		String underscore = "_";
		String langFolder = newLanguage + "/";
		if (newLanguage == null || newLanguage.length() == 0) {
			underscore = "";
			langFolder = "";
		}
		//======= 1. Check resource inside jar
		try {
			result = new PropertyResourceBundle(JREBridge.class.getResourceAsStream("/resources/" + langFolder + bundleName + underscore + newLanguage + ".properties"));
			//languageBundle = new  PropertyResourceBundle(Files.newInputStream(Paths.get(
			//		"resources/PeaLanguagesBundle_" + language + ".properties")));
			
		} catch (NullPointerException e) {
			TestLog.v(JREBridge.class, "Resource Bundle " + langFolder + bundleName + underscore + newLanguage + " not found: " + e.getMessage());
		} catch (IOException e) {
			TestLog.v(JREBridge.class, "Resource Bundle  " + langFolder + bundleName + underscore + newLanguage + " no access: " + e.getMessage());
		} catch (Exception e) {
			TestLog.v(JREBridge.class, "Resource Bundle  " + langFolder + bundleName + underscore + newLanguage + " - unexpected error - " + e.getMessage());
		}
		if (result != null) {
			TestLog.v(JREBridge.class, "Resource bundle inside jar: " + newLanguage);
		}
		return result;
	}

	

	public static void main(String[] args) {
		// TODO Auto-generated method stub

	}

}
