package cologne.eck.all_peas.gui;

import cologne.eck.all_peas.control.PathFileManager;
import cologne.eck.all_peas.data.PeaProperties;
import cologne.eck.file_pea.LockFrameFile;
import cologne.eck.tools.Converter;
import cologne.eck.tools.TestLog;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.KeyStroke;
import javax.swing.Timer;

import java.awt.Window;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.time.Duration;
import java.util.HashSet;


@SuppressWarnings("serial")
public class TimeoutSetter extends JDialog implements ChangeListener, ActionListener {
	
	private static Timer timer = null;
	private static Duration lastDuration = Duration.ZERO;
	
	//private static int timeout = 0;
	private static Duration seconds = Duration.ZERO;
	private static Duration minutes = Duration.ZERO;
	private static Duration hours = Duration.ZERO;
	
	private JSlider secondsSlider = null;
	private JSlider minutesSlider = null;
	private JSlider hoursSlider = null;
	
	private JLabel showTimeout = null;
	
	private static LockFrameFile lff = null;
	
	/**
	 * Open a dialog to set 
	 * @param owner
	 */
	public TimeoutSetter(Window owner) {
		
		if (owner != null && owner instanceof LockFrameFile) {
			lff = (LockFrameFile) owner;
		}

		this.setIconImage(PswDialogView.getImage());
		this.setTitle(PeaProperties.getVmBridge().translate("timer"));

		JPanel contentPane = (JPanel) this.getContentPane();//new JPanel();
		contentPane.setLayout(new BoxLayout(contentPane, BoxLayout.PAGE_AXIS));
		contentPane.setBorder(PeaBorderFactory.createBorder(false));
		
		// avoid keyboard trap: close with escape
		this.getRootPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW)
        	.put(KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0), "CLOSE_ON_ESCAPE");
		this.getRootPane().getActionMap().put("CLOSE_ON_ESCAPE", new CloseOnEscapeAction() );
		
		JPanel infoPanel = new JPanel();
		infoPanel.setLayout(new BoxLayout(infoPanel, BoxLayout.LINE_AXIS));
		JLabel infoLabel = new JLabel(PeaProperties.getVmBridge().translate("timer_tooltip"));
		infoPanel.add(Box.createHorizontalStrut(10));
		infoPanel.add(infoLabel);
		infoPanel.add(Box.createHorizontalGlue());
		contentPane.add(infoPanel);
		
		contentPane.add(Box.createVerticalStrut(20));
		
		int s_MIN = 0;
		int s_MAX = 59;
		int s_INIT = (int) (lastDuration.getSeconds() % 60);    //initial frames per second
		secondsSlider = new JSlider(JSlider.HORIZONTAL,
		                                      s_MIN, s_MAX, s_INIT);
		secondsSlider.setMajorTickSpacing(10);
		secondsSlider.setMinorTickSpacing(1);
		secondsSlider.setPaintTicks(true);
		secondsSlider.setPaintLabels(true);
		secondsSlider.addChangeListener(this);
		JLabel secondsLabel = new JLabel(PeaProperties.getVmBridge().translate("seconds") + ": ");
		secondsLabel.setLabelFor(secondsSlider);
		contentPane.add(secondsLabel);
		contentPane.add(secondsSlider);
		contentPane.add(Box.createVerticalStrut(10));
		
		int m_MIN = 0;
		int m_MAX = 59;
		int m_INIT = (int) ((lastDuration.getSeconds() % 3600) / 60);    //initial frames per second
		minutesSlider = new JSlider(JSlider.HORIZONTAL,
		                                      m_MIN, m_MAX, m_INIT);
		minutesSlider.setMajorTickSpacing(10);
		minutesSlider.setMinorTickSpacing(1);
		minutesSlider.setPaintTicks(true);
		minutesSlider.setPaintLabels(true);
		minutesSlider.addChangeListener(this);
		JLabel minutesLabel = new JLabel(PeaProperties.getVmBridge().translate("minutes") + ": ");
		minutesLabel.setLabelFor(minutesSlider);
		contentPane.add(minutesLabel);
		contentPane.add(minutesSlider);
		contentPane.add(Box.createVerticalStrut(10));
		
		int h_MIN = 0;
		int h_MAX = 25;
		int h_INIT = (int) (lastDuration.getSeconds() / 3600);    //initial frames per second
		hoursSlider = new JSlider(JSlider.HORIZONTAL,
		                                      h_MIN, h_MAX, h_INIT);
		hoursSlider.setMajorTickSpacing(10);
		hoursSlider.setMinorTickSpacing(1);
		hoursSlider.setPaintTicks(true);
		hoursSlider.setPaintLabels(true);
		hoursSlider.addChangeListener(this);
		JLabel hoursLabel = new JLabel(PeaProperties.getVmBridge().translate("hours") + ": ");
		hoursLabel.setLabelFor(hoursSlider);
		contentPane.add(hoursLabel);
		contentPane.add(hoursSlider);
		contentPane.add(Box.createVerticalStrut(20));
		
		JPanel showTimeoutPanel = new JPanel();
		showTimeoutPanel.setLayout(new BoxLayout(showTimeoutPanel, BoxLayout.LINE_AXIS));
		JLabel setLabel = new JLabel(PeaProperties.getVmBridge().translate("exit_program_in"));
		setLabel.setLabelFor(showTimeout);
		showTimeoutPanel.add(setLabel);
		showTimeoutPanel.add(Box.createHorizontalStrut(10));
		showTimeout = new JLabel(getTimeoutAsString(lastDuration));
		showTimeoutPanel.add(showTimeout);
		contentPane.add(showTimeoutPanel);
		contentPane.add(Box.createVerticalStrut(10));
		
		JButton okButton = new JButton(PeaProperties.getVmBridge().translate("ok"));
		okButton.addActionListener(this);
		contentPane.add(okButton);
		
		this.pack();		
		this.setLocationRelativeTo(owner);//.setLocation(MainView.getFrameLocation());
		this.setVisible(true);
	}

	@Override
	public void stateChanged(ChangeEvent e) {
		if (e.getSource() instanceof JSlider) {

			JSlider source = (JSlider)e.getSource();
			if (source == secondsSlider) {
				if (!source.getValueIsAdjusting()) {
					int s = (int)source.getValue();
					seconds = Duration.ofSeconds(s);
					showTimeout.setText(getTimeoutAsString(getTimeoutDuration()));
				}
			} else if (source == minutesSlider) {
				if (!source.getValueIsAdjusting()) {
					int s = (int)source.getValue();
					minutes = Duration.ofMinutes(s);
					showTimeout.setText(getTimeoutAsString(getTimeoutDuration()));
				}
			} else if (source == hoursSlider) {
				if (!source.getValueIsAdjusting()) {
					int s = (int)source.getValue();
					hours = Duration.ofHours(s);
					showTimeout.setText(getTimeoutAsString(getTimeoutDuration()));
				}
			} 
		}
		
	}
	private static String getTimeoutAsString(Duration d) {
		//Duration d = getTimeoutDuration();
		long t = d.getSeconds();
		return String.format("%d:%02d:%02d", t / 3600, (t % 3600) / 60, (t % 60));
		//return String.format("%2d:%02d:%02d", hours.getSeconds() / 3600, minutes.getSeconds() / 60, seconds.getSeconds());
	}


	private static Duration getTimeoutDuration() {
		Duration d = Duration.ZERO;
		d = d.plus(seconds).plus(minutes).plus(hours);
		return d;
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() instanceof JButton) {
			lastDuration = TimeoutSetter.getTimeoutDuration();
			 int timerTimeInMilliSeconds = 5000;
			    timer = new javax.swing.Timer(timerTimeInMilliSeconds, this);
			    timer.start();
			LockFrameFile.setTimer(timer);
			lff.setTimeoutTime(getTimeoutAsString(lastDuration));
			TestLog.v(getClass(), "Timeout: " + getTimeoutAsString(lastDuration));
			this.dispose();
		} else { // timer
			lastDuration = lastDuration.minusSeconds(5);
			if (lastDuration.isNegative()) {
				// avoid asking to remember new files before closing:
				// always remember if program is closed by timeout:
				String[] currFileNames = lff.getCurrentFileNames();
				if (currFileNames != null) {
					HashSet<String> availableSet = Converter.arrayToHashSet(currFileNames);
					PathFileManager.addFileNamesToPathFile(availableSet, false);
				}
				lff.closeProgram();
			}			
			//TestLog.v(getClass(), lastDuration.toString());
			lff.setTimeoutTime(getTimeoutAsString(lastDuration));			
		}		
	}

	public static void setLockFrame(LockFrameFile frame) {
		lff = frame;		
		lff.setTimeoutTime(getTimeoutAsString(lastDuration));
	}
}
