package cologne.eck.image_pea;

/*
 * Peafactory - Production of Password Encryption Archives
 * Copyright (C) 2015  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */

/**
 * Settings for ImageType: 
 *    - WIDTH and HEIGHT of displayed image in LockFrameImage
 *    - RESIZE (keep ratio of width and height) or not. 
 *    
 * Values are set in peagen.ImageType, 
 *   appended as bytes[] to the plain image, 
 *   read as settings in ImageControl
 *   and determine the image displayed in peas.LockFrameImage
 */

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Toolkit;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.EmptyBorder;
import javax.swing.border.LineBorder;

import cologne.eck.all_peas.data.PeaProperties;
import cologne.eck.tools.TestLog;

@SuppressWarnings("serial")
class ImageSetting extends JDialog implements ActionListener {	
	
	private ImageSetting imageSetting;
	private JTextField imageWidthField;
	private JTextField imageHeightField;
	private JCheckBox changeSizeCheck;
	
	public ImageSetting(Window parent, String fileName) {
		//super(PeaFactory.getFrame() );
		imageSetting = this;
		//imageSetting.setUndecorated(true);
		imageSetting.setAlwaysOnTop(true);
		imageSetting.setModal(true);
		
		//necessary for PEA: otherwise file will be overwritten
		imageSetting.setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
		
		if (fileName != null) {
			imageSetting.setTitle(fileName);
		}

		JPanel imagePanel = (JPanel) imageSetting.getContentPane();//new JPanel();
		imagePanel.setBorder(new LineBorder(Color.GRAY,2));
		imagePanel.setLayout(new BoxLayout(imagePanel, BoxLayout.Y_AXIS));
		imagePanel.setBorder(new EmptyBorder(5, 5, 5, 5));		
		imagePanel.add(Box.createVerticalStrut(10));
		
		JPanel labelPanel = new JPanel();
		labelPanel.setLayout(new BoxLayout(labelPanel, BoxLayout.X_AXIS));
		JLabel imageLabel = new JLabel(
				PeaProperties.getVmBridge().translate("settings_for_images"));
		imageLabel.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 16));
//		imageLabel.setPreferredSize(new Dimension(400, 30));
		labelPanel.add(imageLabel);
		labelPanel.add(Box.createHorizontalGlue());
		imagePanel.add(labelPanel);
		imagePanel.add(Box.createVerticalStrut(5));
		
		JPanel tipPanel = new JPanel();
		tipPanel.setLayout(new BoxLayout(tipPanel, BoxLayout.X_AXIS));
		JLabel tipLabel1 = new JLabel(
				PeaProperties.getVmBridge().translate("settings_for_this_session_only"));
		tipLabel1.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 11));
		//tipLabel1.setPreferredSize(new Dimension(400, 50));
		tipPanel.add(Box.createHorizontalStrut(20));
		tipPanel.add(tipLabel1);
		tipPanel.add(Box.createHorizontalGlue());
		imagePanel.add(tipPanel);
		imagePanel.add(Box.createVerticalStrut(15));
		
		JPanel screenPanel = new JPanel();
		screenPanel.setLayout(new BoxLayout(screenPanel, BoxLayout.X_AXIS));
		JLabel screenLabel = new JLabel(
				PeaProperties.getVmBridge().translate("values_of_this_screen"));
		//screenLabel.setPreferredSize(new Dimension(400, 30));
		screenPanel.add(screenLabel);
		screenPanel.add(Box.createHorizontalGlue());
		imagePanel.add(screenPanel);
		
		JPanel screenValuePanel = new JPanel();
		screenValuePanel.setLayout(new BoxLayout(screenValuePanel, BoxLayout.X_AXIS));
		JLabel screenValueLabel = new JLabel(
				"     " + PeaProperties.getVmBridge().translate("width") + ": " 
				+ Toolkit.getDefaultToolkit().getScreenSize().getWidth() 
				+ "   " + PeaProperties.getVmBridge().translate("height") + ": " +  
				+ Toolkit.getDefaultToolkit().getScreenSize().getHeight());
		screenValueLabel.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		screenValuePanel.add(Box.createHorizontalStrut(20));
		screenValuePanel.add(screenValueLabel);
		screenValuePanel.add(Box.createHorizontalGlue());
		imagePanel.add(screenValuePanel);
		
		if (fileName != null) {
			
	    	int width   = 0; // width of the image
	    	int height  = 0; // height of the image
			
			BufferedImage bimg;
			try {
				bimg = ImageIO.read(new File(fileName));
		    	width          = bimg.getWidth();
		    	height         = bimg.getHeight();
			} catch (IOException e) {
				JOptionPane.showMessageDialog(imageSetting, 
						PeaProperties.getVmBridge().translate("invalid_image_file"));
				TestLog.e(getClass(), "ImageSetting - Can't open image file: " + fileName);
				TestLog.e(getClass(), e.toString());
				if (parent instanceof JFrame){ // called in PeaFactory, Menu
					return;
				} else { // called in PEA, initialization
					PeaProperties.getVmBridge().exitWithError(null, "ImageSetting: Inappropriate parent window", true);
				}
			} catch (Exception e) {
				JOptionPane.showMessageDialog(imageSetting, 
						PeaProperties.getVmBridge().translate("unexpected_error")
						+ "\n" + PeaProperties.getVmBridge().translate("invalid_image_file"));
				TestLog.e(getClass(), "ImageSetting - Can't open image file: " + fileName);
				TestLog.e(getClass(), e.toString());
				if (parent instanceof JFrame){ // called in PeaFactory, Menu
					return;
				} else { // called in PEA, initialization
					PeaProperties.getVmBridge().exitWithError(null, "ImageSetting: Inappropriate parent window", true);
				}
			} 
			imagePanel.add(Box.createVerticalStrut(15));
			
			JPanel filePanel = new JPanel();
			filePanel.setLayout(new BoxLayout(filePanel, BoxLayout.X_AXIS));
			JLabel fileLabel = new JLabel(
					PeaProperties.getVmBridge().translate("values_of_images"));
			//fileLabel.setPreferredSize(new Dimension(400, 30));
			filePanel.add(fileLabel);
			filePanel.add(Box.createHorizontalGlue());
			imagePanel.add(filePanel);			
			
			JPanel fileValuePanel = new JPanel();
			fileValuePanel.setLayout(new BoxLayout(fileValuePanel, BoxLayout.X_AXIS));
			JLabel fileValueLabel = new JLabel(
					"     " + PeaProperties.getVmBridge().translate("width") + ": " + width 
					+ "   " + PeaProperties.getVmBridge().translate("height") + ": " + height);
			fileValueLabel.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
			fileValuePanel.add(Box.createHorizontalStrut(20));
			fileValuePanel.add(fileValueLabel);
			fileValuePanel.add(Box.createHorizontalGlue());
			imagePanel.add(fileValuePanel);			
		} 
		imagePanel.add(Box.createVerticalStrut(15));		
		
		JPanel changeSizePanel = new JPanel();
		changeSizePanel.setLayout(new BoxLayout(changeSizePanel, BoxLayout.X_AXIS));
		changeSizeCheck = new JCheckBox(PeaProperties.getVmBridge().translate("change_size"), true);
		changeSizePanel.add(changeSizeCheck);
		changeSizePanel.add(Box.createHorizontalGlue());
		imagePanel.add(changeSizePanel);
		imagePanel.add(Box.createVerticalStrut(10));
					
		JPanel widthPanel = new JPanel();
		widthPanel.setLayout(new BoxLayout(widthPanel, BoxLayout.X_AXIS));		
		JLabel widthLabel = new JLabel(PeaProperties.getVmBridge().translate("width_to_show") + ": " );
		widthLabel.setLabelFor(imageWidthField);
		widthPanel.add(widthLabel);
		imageWidthField = new NumberTextField();
		imageWidthField.setMaximumSize(new Dimension(80, 20));
		imageWidthField.setText("800");
		imageWidthField.setColumns(5);
		imageWidthField.setDragEnabled(true);
		widthPanel.add(Box.createHorizontalStrut(10));
		widthPanel.add(imageWidthField);
		widthPanel.add(Box.createHorizontalGlue());

		imagePanel.add(widthPanel);
		
		JPanel heightPanel = new JPanel();
		heightPanel.setLayout(new BoxLayout(heightPanel, BoxLayout.X_AXIS));
		JLabel heightLabel = new JLabel(PeaProperties.getVmBridge().translate("height_to_show") + ": ");
		heightLabel.setLabelFor(imageHeightField);
		heightPanel.add(heightLabel);
		imageHeightField = new NumberTextField();
		imageHeightField.setMaximumSize(new Dimension(80, 20));
		imageHeightField.setText("600");
		imageHeightField.setColumns(5);
		imageHeightField.setDragEnabled(true);
		heightPanel.add(Box.createHorizontalStrut(10));
		heightPanel.add(imageHeightField);
		heightPanel.add(Box.createHorizontalGlue());

		imagePanel.add(heightPanel);
		imagePanel.add(Box.createVerticalStrut(15));
		
		JPanel resizeInfoPanel = new JPanel();
		resizeInfoPanel.setLayout(new BoxLayout(resizeInfoPanel, BoxLayout.X_AXIS));
		JLabel resizeInfoLabel = new JLabel(PeaProperties.getVmBridge().translate("resize_image"));
		resizeInfoLabel.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 13));
		resizeInfoPanel.add(resizeInfoLabel);
		resizeInfoPanel.add(Box.createHorizontalGlue());
		imagePanel.add(resizeInfoPanel);
		imagePanel.add(Box.createVerticalStrut(10));
				
		JPanel buttonPanel = new JPanel();
		buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
		JButton imageOkButton = new JButton("ok");
		imageOkButton.setActionCommand("newImageSetting");
		imageOkButton.addActionListener(this);
		buttonPanel.add(Box.createHorizontalGlue());
		buttonPanel.add(imageOkButton);
		imagePanel.add(buttonPanel);
	//	imageSetting.setSize(new Dimension(500,300));
		imageSetting.pack();
		imageSetting.setLocation(100, 100);
		imageSetting.setVisible(true);
	}

	@Override
	public void actionPerformed(ActionEvent ape) {
		if (ape.getActionCommand().equals("newImageSetting")) {

			ImageControl.setImageWidth( Integer.parseInt(imageWidthField.getText()) );
			ImageControl.setImageHeight( Integer.parseInt(imageHeightField.getText()) );
			 imageSetting.dispose();
		}
	}
	
	private class NumberTextField extends JTextField {
		private static final long serialVersionUID = 1L;
		public void processKeyEvent(KeyEvent ev) {
		    char c = ev.getKeyChar();
		    try {
		      // printable characters
		    	if (c > 31 && c < 65535 && c != 127) {
		        Integer.parseInt(c + "");// parse
		      }
		      super.processKeyEvent(ev);
		    }
		    catch (NumberFormatException nfe) {
		      // if not a number: ignore 
		    }
		}
	}
}
