package cologne.eck.image_pea;

import java.awt.Dimension;
import java.awt.DisplayMode;
import java.awt.Graphics2D;
import java.awt.GraphicsDevice;
import java.awt.GraphicsEnvironment;
import java.awt.Image;
import java.awt.Point;
import java.awt.event.KeyEvent;
import java.awt.image.BufferedImage;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.AbstractAction;
import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.KeyStroke;
import javax.swing.Timer;

import cologne.eck.all_peas.data.PeaProperties;
import cologne.eck.all_peas.gui.CloseOnEscapeAction;
import cologne.eck.all_peas.gui.IconManager;
import cologne.eck.all_peas.gui.Toaster;
import cologne.eck.all_peas.vm_specific.JREProperties;
import cologne.eck.tools.TestLog;

class Slideshow implements ActionListener {
	
	// time between images in slideshow
	private static int delay = 5000;
	// maximized frame for slideshow
	private static JFrame slideshowFrame = null;
	// dimension of screen, if there are more screen, the smallest one
	private static Dimension screenDim = null;
	// the label that holds the image in the slideshow
	private JLabel slideLabel = null;
	// the timer to change the image in the slideshow
	private Timer timer = null;
	// to check if slideshow and timer is running
	private static boolean timerIsRunning = false;
	// the LockFrameImage instance for that the slideshow was instantiated:
	private LockFrameImage lockFrame = null;
	
	// show initial Toast only once
	private static boolean initToastShown = false;
	// show Toast for keys only limited times:
	private static int stopToastShown = 0;
	// times to show Toasts
	private static final int MAX_SHOWN = 2;
	
	/**
	 * Create a slideshow, 
	 * start with startSlideshow()
	 * 
	 * @param _lockFrame	the current instance of LockFrameImage
	 */
	protected Slideshow(LockFrameImage _lockFrame) {
		
		this.lockFrame = _lockFrame;
	}
	
	/**
	 * Start the slideshow
	 */
	protected void startSlideshow() {
		
		if (slideshowFrame != null && slideshowFrame.isShowing()) {//timerIsRunning == true) {
			TestLog.ve(getClass(), "Slideshow is already running...");
			timerIsRunning = true;
			return;
		}

		timer = new Timer(delay, this);
		timer.setInitialDelay(0);
		timer.setActionCommand("slideshowTimer");
		timer.start(); 
		showCurrentImageAsSlide();		
		timerIsRunning = true;
		if (slideshowFrame != null && initToastShown == false) {
			Toaster.toast(slideshowFrame, 
					PeaProperties.getVmBridge().translate("space_key") + ": " 
					+ PeaProperties.getVmBridge().translate("stop") + "/" + PeaProperties.getVmBridge().translate("go_on")
					+ "\n\n"
					+ PeaProperties.getVmBridge().translate("right_arrow_key") + ": " 
					+ PeaProperties.getVmBridge().translate("next")
					+ "\n\n"
					+ PeaProperties.getVmBridge().translate("left_arrow_key") + ": " 
					+ PeaProperties.getVmBridge().translate("previous")
					, 4); // seconds to show
			initToastShown = true;
		}
	}
	
	/**
	 * Stop the slideshow: stop the timer, 
	 * dispose the frame
	 */
	private void stopSlideshow() {
		if (slideshowFrame != null) {
			slideshowFrame.removeAll();
			slideshowFrame.dispose();
			slideshowFrame = null;
			slideLabel = null;
		}				
		timer.stop();
		timerIsRunning = false;
		return;
	}
	
	/**
	 * Show the current image of LockFrameImage
	 * as image in slideshow
	 */
	private void showCurrentImageAsSlide() {		

		if (slideshowFrame == null) {
			if (timerIsRunning == true) { // was closed with ESC
				stopSlideshow();
				return;
			}
			slideshowFrame = new JFrame();
			Image image = IconManager.loadIcon("pea-lock.png", 
					PeaProperties.getVmBridge().translate("pea_with_keyhole"), 
					null, 
					PeaProperties.getVmBridge().translate("logo_of_pea") ).getImage();
			slideshowFrame.setIconImage(image);
			slideLabel = new JLabel();
			slideshowFrame.getContentPane().add(slideLabel);
			slideLabel.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW)
	        .put(KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0), "CLOSE_ON_ESCAPE");
			slideLabel.getActionMap().put("CLOSE_ON_ESCAPE", new CloseOnEscapeAction() );
			
			slideLabel.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW)
	        .put(KeyStroke.getKeyStroke(KeyEvent.VK_RIGHT, 0), "NEXT");
			slideLabel.getActionMap().put("NEXT", new NextAction() );
			slideLabel.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW)
	        .put(KeyStroke.getKeyStroke(KeyEvent.VK_LEFT, 0), "PREVIOUS");
			slideLabel.getActionMap().put("PREVIOUS", new PreviousAction() );
			slideLabel.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW)
	        .put(KeyStroke.getKeyStroke(KeyEvent.VK_SPACE, 0), "STOP");
			slideLabel.getActionMap().put("STOP", new StopAction() );
			
			slideshowFrame.setExtendedState(JFrame.MAXIMIZED_BOTH);
			slideshowFrame.setResizable(false);
			slideshowFrame.setLocation(new Point(0,0));
			slideshowFrame.setVisible(true);
		} else {		
			//
		}
		if (screenDim == null) { // compute once:
			// get the smaller one of two screens
			GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
			GraphicsDevice[] gs = ge.getScreenDevices();
			double area = 0;
			DisplayMode dm = null;
			for(GraphicsDevice curGs : gs) {
			      dm = curGs.getDisplayMode();
			      if (dm.getWidth() * dm.getHeight() < area || area == 0) {
			    	  screenDim = new Dimension(dm.getWidth(), dm.getHeight());
			      }
			}
			if (screenDim == null) {
				screenDim = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
			}
		}
		
		BufferedImage image = null;

		// check if language was change, the slideshow can than not be closed with ESC
		if (JREProperties.getMainWindow() instanceof LockFrameImage) {
			LockFrameImage currentLockFrame = (LockFrameImage) JREProperties.getMainWindow();
			if ( ! currentLockFrame.equals(lockFrame)) {
				// language changed
				stopSlideshow();
				return;
			} 
			image = lockFrame.getImage();
		}
		if (image == null) {
			TestLog.ve(getClass(), "Cant get image", 5);
			return;
		}				
		// new compressed/resized image
		int[] imageSize = ImageHelper.resizeImage(image, (int) screenDim.getWidth(), (int) screenDim.getHeight());

		BufferedImage  compressedImage = new BufferedImage(
				(int) screenDim.getWidth(), (int) screenDim.getHeight(),
              BufferedImage.TYPE_INT_ARGB);
        Graphics2D g = compressedImage.createGraphics();
        g.drawImage(image, 0, 0, imageSize[0], imageSize[1], lockFrame);              
        g.dispose();		

        if (slideLabel != null && compressedImage != null) {
        	slideLabel.setIcon(new ImageIcon(compressedImage));
        } else {
        	TestLog.ve(getClass(), "Label is null: " + (slideLabel == null) + ", image is null: " + (compressedImage == null), 5);
        	stopSlideshow();
        }
	}

	@Override
	public void actionPerformed(ActionEvent ae) {

		String command = ae.getActionCommand();

		if (command.equals("slideshowTimer")){
			
			LockFrameImage currentLockFrame = (LockFrameImage) JREProperties.getMainWindow();
			if ( ! currentLockFrame.equals(lockFrame)) { // language was changed, slideshow can not be stopped by ESC
				stopSlideshow();
				return;
			}
			if ( (slideshowFrame == null || slideshowFrame.isShowing() == false) && timerIsRunning == true) { // was closed by ESC or close
				stopSlideshow();
				return;
			}
			lockFrame.goToNextImage();
			showCurrentImageAsSlide();
		}
	}
	
	/**
	 * Set the delay of the slideshow
	 * 
	 * @param _delay	delay (in seconds)
	 */
/*	public static void setDelaySeconds(int _delay) { // TODO image.properties file and ImageProperties class
		if (_delay == 0 || _delay < 0) {
			TestLog.ve(Slideshow.class,  "Invalid delay: " + _delay);
			return;
		}
		Slideshow.delay = (_delay * 1000);
	} */
	
	//======== Inner classes: =========================================
	
	@SuppressWarnings("serial")
	private class NextAction extends AbstractAction {

		@Override
		public void actionPerformed(ActionEvent arg0) {
			lockFrame.goToNextImage();
			showCurrentImageAsSlide();
			timer.stop();
			timerIsRunning = false;
			if (slideshowFrame != null && stopToastShown < MAX_SHOWN) {
				Toaster.toast(slideshowFrame, 
						PeaProperties.getVmBridge().translate("space_key") + ": " 
								+ PeaProperties.getVmBridge().translate("go_on"), 1);
				stopToastShown++;
			}
		}		
	}
	
	@SuppressWarnings("serial")
	private class PreviousAction extends AbstractAction {

		@Override
		public void actionPerformed(ActionEvent arg0) {
			lockFrame.goToPreviousImage();
			showCurrentImageAsSlide();
			timer.stop();
			timerIsRunning = false;
			if (slideshowFrame != null && stopToastShown < MAX_SHOWN) {
				Toaster.toast(slideshowFrame, 
						PeaProperties.getVmBridge().translate("space_key") + ": " 
								+ PeaProperties.getVmBridge().translate("go_on"), 1);
				stopToastShown++;
			}
		}		
	}
	
	@SuppressWarnings("serial")
	private class StopAction extends AbstractAction {

		@Override
		public void actionPerformed(ActionEvent arg0) {
			if (timerIsRunning == true) {
				timer.stop();
				timerIsRunning = false;
				if (slideshowFrame != null && stopToastShown < MAX_SHOWN) {
					Toaster.toast(slideshowFrame, 
							PeaProperties.getVmBridge().translate("space_key") + ": " 
									+ PeaProperties.getVmBridge().translate("go_on"), 1);
					stopToastShown++;
				}
			} else {
				timer.start();
				timerIsRunning = true;
			}
			/* lockFrame.goToPreviousImage();
			showCurrentImageAsSlide(); */
		}		
	}
	
}
